#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Baralho */
#include "baralho.h"

/* Graficos */
#include "grafico.h"

/* Modulos do programa */
#include "ihm.h"
#include "jogo.h"
#include "disco.h"

//-----------------------------------------------------------------------------

MonteImagem *iniPosMontes(Window win, Cabecalho cartaDim)
{
  int i;
  Coord montesBaseCoord;
  MonteImagem *montes;
 
  /* Posio Inicial dos montes de cartas */
  montesBaseCoord.x = (win.disp_data.width - 
                      (PILHAS_CARTAS*cartaDim.w + 
		       (PILHAS_CARTAS-1)*PILHAS_CARTAS_INTERVALO))/2;
  montesBaseCoord.y = win.disp_data.height / 4;

  /* Inicializa os montes (sem cartas) */
  montes = (MonteImagem *) calloc (PILHAS_CARTAS+2, sizeof(MonteImagem));

  for (i=0; i<PILHAS_CARTAS+1; ++i) {
    montes[i].topoImagem.bmp = imagemPPM(NULL, cartaDim.w, cartaDim.h, win);
    if (i) {
      montes[i].topoImagem.x = montesBaseCoord.x + i*(cartaDim.w + PILHAS_CARTAS_INTERVALO);
      montes[i].topoImagem.y = montesBaseCoord.y;
    }
    else {
      montes[i].topoImagem.x = (win.disp_data.width - cartaDim.w)/2;
      montes[i].topoImagem.y = win.disp_data.height - cartaDim.h - 200;
    }
    montes[i].topoImagem.w = cartaDim.w;
    montes[i].topoImagem.h = cartaDim.h;
  }

  return montes;
}

void jogarJogo(Baralho cartas, Cabecalho cartaDim,
	       MonteImagem *montes, Window win)
{

  bool redraw, sair;
  int i, cartaMonteOrg;
  Carta cartaDeck, cartaMonte;

/* Mostra comandos de mouse na tela */
  printf("Botao esquerdo: Mover carta entre montes\n");
  printf("Botao direito: Embaralha deck principal\n");
  printf("Botao do meio: Embaralha os montes, exceto o deck principal.\n");
  printf("Tecla <ESC>: Sair do programa\n");
  printf("Tecla <F1>: Salva estado dos montes em disco\n");
 
  /* Trata eventos enquanto  a tecla ESC  no for  pressionada. */

  redraw = true;
  sair = false;

  while (!sair) {
    ALLEGRO_EVENT ev;
    al_wait_for_event(win.event_queue, &ev);
    
    if(ev.type == ALLEGRO_EVENT_DISPLAY_CLOSE) {
      sair=true;
    }
    else if(ev.type == ALLEGRO_EVENT_TIMER) {
      redraw = true;
    }
    else if(ev.type == ALLEGRO_EVENT_MOUSE_BUTTON_DOWN) {
      switch(ev.mouse.button){
      case 1: /* verifica qual monte est sob o ponteiro do mouse ao apertar
		 o boto
	      */
	      for (i=0; montes[i].topoImagem.bmp; ++i)
		if (is_inside_bitmap(montes[i].topoImagem, ev.mouse.x, ev.mouse.y)) {
		  cartaMonteOrg = i;
		  break;
		}
	      break;
      case 2:  montes[0].cartas = embaralhaMonte(montes[0].cartas);
	       break;
      case 3:  for (i=1; montes[i].topoImagem.bmp; ++i)
		 montes[i].cartas = embaralhaMonte(montes[i].cartas);
	       break;
      }
   }
    else if(ev.type == ALLEGRO_EVENT_MOUSE_BUTTON_UP) {
      switch(ev.mouse.button){
      case 1: /* verifica qual monte est sob o ponteiro do mouse ao soltar
		 o boto
	      */
	       for (i=1; montes[i].topoImagem.bmp; ++i) {
		 if (is_inside_bitmap(montes[i].topoImagem, ev.mouse.x, ev.mouse.y)) {
		   // tira carta do monte de origem 
		   cartaMonte = tiraCarta(montes[cartaMonteOrg].cartas, cartas);
		   // coloca no monte de destino
		   montes[i].cartas = colocaCarta(cartaMonte, montes[i].cartas);
		   break;
		 }
	       }
	       break;
      }
    }
    else if(ev.type == ALLEGRO_EVENT_KEY_UP) {
      // Detecta se apertaram a tecla ESC
      switch(ev.keyboard.keycode) {
      case ALLEGRO_KEY_ESCAPE: sair = true; break;
      case ALLEGRO_KEY_F1: salvarEstadoJogo(montes, cartas);
	                   break;
      }
    }	    

    /* Se no h eventos a tratar e se o Timer acabou, redesenha
       bitmaps no display
    */
    if(redraw && al_is_event_queue_empty(win.event_queue)) {
      redraw = false; // Fica true quando Timer acaba

      // Cor do funda da janela (display)
      al_clear_to_color(VERDE);
      
      /* mostra as cartas do topo de cada monte */
      for (i=0; montes[i].topoImagem.bmp; ++i) {
	/* Se h cartas no monte, mostra a carta do topo. 
	   Se no h, mostra fundo branco.
	*/
	if (!monteVazio(montes[i].cartas))
	  cartaMonte = topoMonte(montes[i].cartas, cartas);
	else {
	  cartaMonte.valor = cartaMonte.naipe = 0;
	  cartaMonte.img  = NULL;
	}
	/* mostra monte alterado */
	mudaImgPPM(montes[i].topoImagem, cartaMonte.img, cartaDim.w, cartaDim.h, win);
      }

      // Atualiza o display para que mudanas sejam visveis
      al_flip_display();
    }
  }
  
  

}

void removeImagens(MonteImagem *montes, Window win)
{
  int i;

  for (i=0; montes[i].topoImagem.bmp; ++i)
    al_destroy_bitmap(montes[i].topoImagem.bmp);

  graphdeinit(win);
}

